;******************************************************************************
;*  GMT - GGLABS MEMORY TEST                                                  *
;*  A modern Apple IIgs memory test optimized for coverage and speed          *
;*  Copyright 2017, 2018 Gabriele Gorla                                       *
;*                                                                            *
;*  This program is free software: you can redistribute it and/or modify      *
;*  it under the terms of the GNU General Public License as published by      *
;*  the Free Software Foundation, either version 3 of the License, or         *
;*  (at your option) any later version.                                       *
;*                                                                            *
;*  GMT is distributed in the hope that it will be useful,                    *
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;*  GNU General Public License for more details.                              *
;*                                                                            *
;*  You should have received a copy of the GNU General Public License         *
;*  along with GMT.  If not, see <http://www.gnu.org/licenses/>.              *
;*                                                                            *
;******************************************************************************
	
.setcpu		"65816"
.autoimport 	on
.importzp	sp

.import 	_err_list
.import 	_DPat
.import 	_NErr

.export 	_Bank_Set
.export 	_Bank_Write
.export 	_Bank_Read

.segment	"CODE"

bankbank:
.byte $00,$00

; ****************** Bank Set *********************
.proc _Bank_Set: near
.I8
.A8
	lda (sp)		; load bank in A
	sta bankbank
	sta bankbank+1          ; reference data bank

 	sta wpatch              ; patch Bank_Write
	sta rpatch		; patch Bank_Read
 	sta rrpatch 		; patch Bank_Read re-read
	jsr incsp1		; restore stak pointer
	rts
.endproc


; ****************** Bank Write *********************
_Bank_Write:
.I8
.A8
	clc			; enable native mode
	xce			;
	rep #$30 		; force m and x to 16-b
.I16
.A16
	lda _DPat
	ldx #0000
wloop:	sta $100000,x
wpatch = * -1
	inx
	inx
	bne wloop

	sec			; back to 6502 emulation
	xce			;
.I8
.A8
	rts

; ****************** Bank Read *********************
_Bank_Read:
.I8
.A8
	clc			; enable native mode
	xce			;
	rep #$30 		; force m and x to 16-b
.I16
.A16
	ldx #$0000
rloop:	lda $100000,x
rpatch = * -1
	cmp _DPat
	bne error
resume:	inx
	inx
	bne rloop		; 
	sec			; back to 6502 emulation
	xce			;
.I8
.A8
	rts

.I16
.A16
error:	pha                     ; push read data
	lda _NErr		; load current number of errors
	and #$000f		; limit to the size fo the array
	asl a			; multiply by 16 (size of one entry)
	asl a
	asl a
	asl a
	tay			; put the entry offset into Y
	txa			; move error address to A
	sta _err_list,y		; store error address into error entry
	lda _DPat		; load expected pattern
	sta _err_list+2,y	; pet expected aptter into error entry
	pla                     ; pull the read data
	sta _err_list+4,y	; store bad read into error entry
	jsr reread		; re-read @ bad address
	sta _err_list+6,y	; store re-read into error entry
	jsr reread
	sta _err_list+8,y
	jsr reread
	sta _err_list+10,y
	inc _NErr		; increment error number
	bra resume		; back to read test

reread:	lda $100000,x
rrpatch = *-1
	rts
